package com.qmth.teachcloud.mark.service.impl;

import com.qmth.teachcloud.mark.entity.MarkQuestion;
import com.qmth.teachcloud.mark.entity.MarkStudent;
import com.qmth.teachcloud.mark.service.MarkPaperService;
import com.qmth.teachcloud.mark.service.MarkQuestionService;
import com.qmth.teachcloud.mark.service.MarkStudentService;
import com.qmth.teachcloud.mark.dto.mark.ScoreItem;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkStepDTO;
import com.qmth.teachcloud.mark.dto.mark.manage.Task;
import com.qmth.teachcloud.mark.dto.mark.manage.TrackDTO;
import com.qmth.teachcloud.mark.entity.MarkArbitrateHistory;
import com.qmth.teachcloud.mark.entity.MarkGroup;
import com.qmth.teachcloud.mark.entity.MarkHeaderTrack;
import com.qmth.teachcloud.mark.service.MarkHeaderTrackService;
import com.qmth.teachcloud.mark.service.TaskService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

@Service
public class TaskServiceImpl implements TaskService {

    @Resource
    private MarkPaperService markPaperService;
    @Resource
    private MarkStudentService markStudentService;
    @Resource
    private MarkQuestionService markQuestionService;
    @Resource
    private MarkHeaderTrackService markHeaderTrackService;

    @Override
    public Task build(MarkArbitrateHistory markArbitrateHistory, MarkGroup markGroup) {
        MarkStudent markStudent = markStudentService.getById(markArbitrateHistory.getStudentId());
        Task task = new Task();
        task.setStudentId(markArbitrateHistory.getStudentId());
        task.setLibraryId(markArbitrateHistory.getId());
        task.setSecretNumber(markArbitrateHistory.getSecretNumber());
        task.setQuestionList(buildArbitrateStep(markGroup, markArbitrateHistory));
//        task.setSpecialTagList(getHeaderTagList(history.getStudentId(), group.getNumber()));
        // todo jsonurl xf
//        task.setJsonUrl(fileService.getJsonUri(student.getExamId(), student.getSecretNumber()));
        if (StringUtils.isNotBlank(markStudent.getObjectiveScoreList())) {
            task.setObjectiveScore(markStudent.getObjectiveScore());
        }
        task.setMarkerTime(markArbitrateHistory.getUpdateTime());
        if (markArbitrateHistory.getTotalScore() != null) {
            task.setMarkerScore(markArbitrateHistory.getTotalScore());
        }
        return task;
    }

    private List<MarkStepDTO> buildArbitrateStep(MarkGroup group, MarkArbitrateHistory markArbitrateHistory) {
        List<MarkStepDTO> list = new LinkedList<>();
        List<MarkQuestion> markQuestionList = markQuestionService.listQuestionByExamIdAndPaperNumberAndGroupNumber(group.getExamId(), group.getPaperNumber(), group.getNumber(), false);
        List<MarkHeaderTrack> tracks = markHeaderTrackService.listByExamAndPaperNumberAndGroupNumberAndStudentId(group.getExamId(), group.getPaperNumber(), group.getNumber(), markArbitrateHistory.getStudentId());
        List<ScoreItem> sItems = new ArrayList<ScoreItem>();
        if (markArbitrateHistory != null) {
            sItems = markArbitrateHistory.getScoreItemList();
        }
        for (int i = 0; i < markQuestionList.size(); i++) {
            MarkQuestion question = markQuestionList.get(i);
            MarkStepDTO step = buildStep(question);
            if (markArbitrateHistory != null) {
                if (!sItems.isEmpty() && sItems.size() == markQuestionList.size()) {
                    step.setScore(sItems.get(i).getScore());
                }
                // 增加阅卷轨迹列表获取
                String questionNumber = question.getQuestionNumber();
                for (MarkHeaderTrack track : tracks) {
                    if (track.getQuestionNumber().equals(questionNumber)) {
                        step.addTrack(new TrackDTO(track));
                    }
                }
            }
            list.add(step);
        }
        return list;
    }

    private MarkStepDTO buildStep(MarkQuestion question) {
        MarkStepDTO step = new MarkStepDTO();
        step.setMainNumber(question.getMainNumber());
        step.setSubNumber(String.valueOf(question.getSubNumber()));
        step.setGroupNumber(question.getGroupNumber());
        step.setTitle(question.getMainTitle());
        step.setDefaultScore(0d);
        step.setMaxScore(question.getTotalScore());
        step.setMinScore(0d);
        step.setIntervalScore(question.getIntervalScore());
        step.setUncalculate(false);
        return step;
    }
}
